#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 21 11:29:13 2025

@author: vsicardi
"""
"""
================================================================================
EC-Earth General Assembly Gender Statistics Script
================================================================================

This script was developed to generate visual summaries of gender representation
among participants, speakers, and invited speakers at EC-Earth General Assembly
(GA) events from 2018 to 2025. 

 The plots produced have been used in the poster presented during the 
EC-Earth GA in March 2025.

⚠️ DISCLAIMER:
- Gender classification has been inferred from speaker/participant names or 
  official program PDFs.
- This introduces the possibility of misclassification or errors.
- The dataset is NOT exhaustive and may contain inaccuracies.
- Contributions and corrections are welcome.

 Script functionality:
- Reads participant and speaker lists from CSV files.
- Aggregates gender-based counts for participants, speakers, and invited talks.
- Produces stacked bar plots comparing gender statistics over events.

 This code is open and in early development. Contributions are highly encouraged.

Author: Valentina Sicardi, Barcelona Supercomputing Center
email: valentina.sicardi@bsc.es
Date: March 2025

================================================================================
"""


import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import os

print(os.getcwd())
os.chdir("/home/vsicardi/WORK/EDI_ecearth")
print(os.getcwd())



#=========== Participants Session =================
gender_counts = {}

# List of available files
csv_files = [
    "Participants_10-2018_InPerson.csv",
    "Participants_05-2019_InPerson.csv",
    "Participants_03-2020_Virtual.csv",
    "Participants_02-2021_Virtual.csv",
    "Participants_11-2021_Virtual.csv",
    "Participants_10-2022_InPerson.csv",
    "Participants_06-2023_Virtual.csv",
    "Participants_03-2024_InPerson.csv",
    "Participants_09-2024_Virtual.csv",
    "Participants_03-2025_InPerson.csv"
]



# Loop through each file and process data
for file in csv_files:
    try:
        df = pd.read_csv(file)
        event_key = os.path.basename(file).replace("Participants_", "").replace(".csv", "")  # Extract "MM-YYYY_Type" format
        
        if "Gender" in df.columns:
            gender_counts[event_key] = {
                "Women": (df["Gender"] == "F").sum(),
                "Men": (df["Gender"] == "M").sum()
            }
    except Exception as e:
        print(f"Error processing {file}: {e}")

# Convert dictionary to DataFrame
plot_df = pd.DataFrame.from_dict(gender_counts, orient="index").fillna(0)

# Define event labels for x-axis
events = np.arange(len(plot_df))

# Create figure
fig, ax = plt.subplots(figsize=(14, 6))
bar_width = 0.6  # Width of bars

# Plot stacked bars
bars_women = ax.bar(events, plot_df["Women"], bar_width, label="Participants (Women)", color="violet")
bars_men = ax.bar(events, plot_df["Men"], bar_width, bottom=plot_df["Women"], label="Participants (Men)", color="orange")

# Formatting
ax.set_title("Number of Participants by Gender in EC-Earth GA", fontsize=20)
ax.set_xlabel("GA", fontsize=18)
ax.set_ylabel("Number of Participants", fontsize=18)
ax.set_xticks(events)
ax.set_xticklabels(plot_df.index, rotation=45, ha='right', fontsize=14)
ax.legend(title="", fontsize=14)
ax.grid(axis="y", linestyle="--", alpha=0.7)

# Custom Y-ticks: Every number up to 10, then every 10
max_y = int(plot_df.values.max()) + 2
y_ticks = list(range(0, min(11, max_y + 1))) + list(range(10, max_y + 1, 10))
#ax.set_yticks(y_ticks)

# Show the plot
plt.show()

#================= speakers and invited speakers 
# List of file paths
file_paths = [
    "Speakers_10-2018_In_Person.csv",
    "Speakers_05-2019_InPerson.csv",
    "Speakers_03-2020_Virtual.csv",
    "Speakers_02-2021_Virtual.csv",
    "Speakers_11-2021_Virtual.csv",
    "Speakers_10-2022_In_Person.csv",
    "Speakers_06-2023_Virtual.csv",
    "Speakers_03-2024_InPerson.csv",
    "Speakers_09-2024_Virtual.csv",
    "Speakers_03-2025_Inperson.csv"
]

# Dictionary to store counts
speaker_data = {}

# Process each file
for file_path in file_paths:
    try:
        df = pd.read_csv(file_path)

        # Extract event name from the filename
        event_name = os.path.basename(file_path).replace("Speakers_", "").replace(".csv", "")

        # Standardize column names
        df.columns = df.columns.str.strip()

        # Count speakers by gender
        if "Gender" in df.columns and "Invited Talk" in df.columns:
            speaker_data[event_name] = {
                "Speakers_Women": ((df["Gender"] == "F") & (df["Invited Talk"] == "No")).sum(),
                "Speakers_Men": ((df["Gender"] == "M") & (df["Invited Talk"] == "No")).sum(),
                "Invited_Women": ((df["Gender"] == "F") & (df["Invited Talk"] == "Yes")).sum(),
                "Invited_Men": ((df["Gender"] == "M") & (df["Invited Talk"] == "Yes")).sum()
            }
    except Exception as e:
        print(f"Error processing {file_path}: {e}")

# Convert dictionary to DataFrame
speaker_df = pd.DataFrame.from_dict(speaker_data, orient="index").fillna(0)

# Plot stacked bar chart
fig, ax = plt.subplots(figsize=(14, 6))
bar_width = 0.3  # Reduced width to differentiate the two bars

# Define event labels for x-axis
events = np.arange(len(speaker_df))

# Plot stacked bars for normal speakers
bars_speakers_women = ax.bar(events - bar_width/2, speaker_df["Speakers_Women"], bar_width, label="Speakers (Women)", color="lightblue")
bars_speakers_men = ax.bar(events - bar_width/2, speaker_df["Speakers_Men"], bar_width, bottom=speaker_df["Speakers_Women"], label="Speakers (Men)", color="blue")

# Plot stacked bars for invited speakers
bars_invited_women = ax.bar(events + bar_width/2, speaker_df["Invited_Women"], bar_width, label="Invited (Women)", color="lightcoral", hatch="//")
bars_invited_men = ax.bar(events + bar_width/2, speaker_df["Invited_Men"], bar_width, bottom=speaker_df["Invited_Women"], label="Invited (Men)", color="red", hatch="//")

# Ensure bars are visible even if very few invited speakers
for i, (x, women, men) in enumerate(zip(events + bar_width/2, speaker_df["Invited_Women"], speaker_df["Invited_Men"])):
    if women + men < 2:  # Ensure visibility for small counts
        ax.text(x, women + men + 0.3, f"{int(women + men)}", ha='center', fontsize=10, fontweight='bold')

# Formatting
ax.set_title("Number of Speakers and Invited Speakers by Gender in EC-Earth GA", fontsize=20)
ax.set_xlabel("GA", fontsize=18)
ax.set_ylabel("Number of Speakers", fontsize=18)
ax.set_xticks(events)
ax.set_xticklabels(speaker_df.index, rotation=45, ha='right',fontsize=14)
ax.legend(title="", fontsize=14)
ax.grid(axis="y", linestyle="--", alpha=0.7)


# Custom Y-ticks: Every number up to 10, then every 10
max_y = int(speaker_df.values.max()) + 2
y_ticks = list(range(0, min(11, max_y + 1))) + list(range(10, max_y + 1, 10))
#ax.set_yticks(y_ticks)


# Show the plot
plt.show()







#######
###### bonus track  plot all the data in the same plot. 
#
# Loop through each file and process data
for file in csv_files:
    try:
        df = pd.read_csv(file)
        event_key = os.path.basename(file).replace("Participants_", "").replace(".csv", "")

        if "Gender" in df.columns:
            gender_counts[event_key] = {
                "Participants_Women": (df["Gender"] == "F").sum(),
                "Participants_Men": (df["Gender"] == "M").sum()
            }
    except Exception as e:
        print(f"Error processing {file}: {e}")

# Convert dictionary to DataFrame
participant_df = pd.DataFrame.from_dict(gender_counts, orient="index").fillna(0)

# Load speaker data
speaker_files = [

    "Speakers_10-2018_In_Person.csv",
    "Speakers_05-2019_InPerson.csv",
    "Speakers_03-2020_Virtual.csv",
    "Speakers_02-2021_Virtual.csv",
    "Speakers_11-2021_Virtual.csv",
    "Speakers_10-2022_In_Person.csv",
    "Speakers_06-2023_Virtual.csv",
    "Speakers_03-2024_InPerson.csv",
    "Speakers_09-2024_Virtual.csv",
    "Speakers_03-2025_Inperson.csv"
]

# Dictionary to store counts
speaker_data = {}

# Process each file
for file_path in speaker_files:
    try:
        df = pd.read_csv(file_path)
        event_name = os.path.basename(file_path).replace("Speakers_", "").replace(".csv", "")

        # Standardize column names
        df.columns = df.columns.str.strip()

        # Count speakers by gender
        if "Gender" in df.columns and "Invited Talk" in df.columns:
            speaker_data[event_name] = {
                "Speakers_Women": ((df["Gender"] == "F") & (df["Invited Talk"] == "No")).sum(),
                "Speakers_Men": ((df["Gender"] == "M") & (df["Invited Talk"] == "No")).sum(),
                "Invited_Women": ((df["Gender"] == "F") & (df["Invited Talk"] == "Yes")).sum(),
                "Invited_Men": ((df["Gender"] == "M") & (df["Invited Talk"] == "Yes")).sum()
            }
    except Exception as e:
        print(f"Error processing {file_path}: {e}")

# Convert dictionary to DataFrame
speaker_df = pd.DataFrame.from_dict(speaker_data, orient="index").fillna(0)

# Merge participant and speaker data
merged_df = participant_df.join(speaker_df, how="outer").fillna(0)

# Plot stacked bar chart
fig, ax = plt.subplots(figsize=(14, 6))
bar_width = 0.3  # Adjusted width to accommodate multiple bars

# Define event labels for x-axis
events = np.arange(len(merged_df))

# Plot participants
ax.bar(events - bar_width, merged_df["Participants_Women"], bar_width, label="Participants (Women)", color="violet")
ax.bar(events - bar_width, merged_df["Participants_Men"], bar_width, bottom=merged_df["Participants_Women"], label="Participants (Men)", color="orange")

# Plot speakers
ax.bar(events, merged_df["Speakers_Women"], bar_width, label="Speakers (Women)", color="lightblue")
ax.bar(events, merged_df["Speakers_Men"], bar_width, bottom=merged_df["Speakers_Women"], label="Speakers (Men)", color="blue")

# Plot invited speakers
ax.bar(events + bar_width, merged_df["Invited_Women"], bar_width, label="Invited (Women)", color="lightcoral", hatch="//")
ax.bar(events + bar_width, merged_df["Invited_Men"], bar_width, bottom=merged_df["Invited_Women"], label="Invited (Men)", color="red", hatch="//")

# Formatting
ax.set_title("Number of Participants, Speakers, and Invited Speakers by Gender in EC-Earth GA Events", fontsize=20)
ax.set_xlabel("GA Events", fontsize=18)
ax.set_ylabel("Number of Participants", fontsize=18)
ax.set_xticks(events)
ax.set_xticklabels(merged_df.index, rotation=45, ha='right', fontsize=14)
ax.legend(title="", fontsize=14)
ax.grid(axis="y", linestyle="--", alpha=0.7)

# Custom Y-ticks: Every number up to 10, then every 10
max_y = int(merged_df.values.max()) + 2
y_ticks = list(range(0, min(11, max_y + 1))) + list(range(10, max_y + 1, 10))
ax.set_yticks(y_ticks)

# Show the plot
plt.show()

